"""Default settings"""
import os.path

APP_NAME = "jyou"
DEBUG_MODE = False

if "XDG_DATA_HOME" in os.environ:
    DATA_PATH = os.path.join(os.getenv("XDG_DATA_HOME"), APP_NAME)
else:
    DATA_PATH = os.path.expanduser("~/.local/share/" + APP_NAME)

if "XDG_CONFIG_HOME" in os.environ:
    CONFIG_PATH = os.path.join(os.getenv("XDG_CONFIG_HOME"), APP_NAME)
else:
    CONFIG_PATH = os.path.expanduser("~/.config/" + APP_NAME)

if "XDG_CACHE_HOME" in os.environ:
    CACHE_PATH = os.path.join(os.getenv("XDG_CACHE_HOME"), APP_NAME)
else:
    CACHE_PATH = os.path.expanduser("~/.cache/" + APP_NAME)



"""File for handling config files"""
import copy
import os
import json
from typing import Dict

import pickle
from jyou import settings

CONFIG_FILE_PATH = os.path.join(settings.CONFIG_PATH, "config.json")

DEFAULT_CONFIG = {
    "blur": 0,
    "brightness": 1,
    "out_directory": "$HOME/.local/share/jyou/",
    "progress": False,
    "debug": False,
}


def parse_config() -> Dict:
    """
    Parses and returns the config file, creating a new one if it does not exist

    Returns:
        (Dict): a dictionary containing the config file
    """
    config = copy.copy(DEFAULT_CONFIG)

    try:
        with open(CONFIG_FILE_PATH, encoding="UTF-8") as config_file:
            loaded_config = json.load(config_file)
            config.update(loaded_config)
    except IOError:
        save_config(config)

    return config


def save_config(config: Dict):
    """
    Saves the config to the config location

    Arguments:
        config (Dict): the config
    """
    config_path = os.path.dirname(CONFIG_FILE_PATH)
    if not os.path.isdir(config_path):
        os.mkdir(config_path)

    with open(CONFIG_FILE_PATH, "wb") as config_file:
        config_file.write(
            json.dumps(config, indent=4, separators=(",", ": ")).encode("utf-8")
        )


def load_config(config_file_path: str) -> Dict:
    """
    Load the config from the specified location

    Arguments:
        config_file_path (str): the path to the config file

    Returns:
        (Dict): the config
    """
    with open(config_file_path, "rb") as config_file:
        return pickle.load(config_file, encoding="utf-8")


def compare_flag_with_config(flag: bool, config_option: bool) -> bool:
    """
    Compares the value of the given flag with the config option, with the flag
    taking more presedence over the conifg

    Arguments:
        flag (bool): the flag
        config_option (bool): the option in the config

    Returns:
        (bool): the result
    """
    if flag:
        return flag

    return config_option



"""Logging functions"""
import logging
import tqdm

DEFAULT_FORMAT = "[%(levelname)s\033[0m] " "\033[1;31m%(module)s\033[0m: " "%(message)s"
BAR_FORMAT = "{percentage:3.0f}% {n}/{total}"


class TqdmLoggingHandler(logging.Handler):
    """A logging handler for tqdm progress stuff"""

    def __init__(self, level=logging.NOTSET):
        super().__init__(level)

    def emit(self, record):
        try:
            self.setFormatter(logging.Formatter(DEFAULT_FORMAT))
            msg = self.format(record)
            tqdm.tqdm.write(msg)
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


class DefaultLoggingHandler(logging.StreamHandler):
    """A logging handler for normal logging"""

    def emit(self, record):
        try:
            self.setFormatter(logging.Formatter(DEFAULT_FORMAT + "\n"))
            msg = self.format(record)
            stream = self.stream
            stream.write(msg)
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


def setup_logger(name, level, handler):
    """Sets up the logger"""
    logger = logging.getLogger(name)
    logger.setLevel(level)
    logger.addHandler(handler)
    return logger


logging.addLevelName(logging.ERROR, "\033[1;31mE")
logging.addLevelName(logging.INFO, "\033[1;32mI")
logging.addLevelName(logging.WARNING, "\033[1;33mW")
logging.addLevelName(logging.CRITICAL, "\033[1;33mC")
logging.addLevelName(logging.DEBUG, "\033[1;33mD")



#!/usr/bin/env python3
"""
JYOU
Author: slapeLachie <lslape@slapelachie.xyz>

Simple lockscreen manager for tiling window managers. Generates a lockscreen for
multiscreened i3lock.
"""

import argparse
import sys
import os.path
import logging
import shutil

from jyou.__init__ import __version__, __author__, __email__
from jyou import log, config_handler
from jyou.settings import DATA_PATH, CONFIG_PATH
from jyou.generator import LockscreenGenerator

logger = log.setup_logger(__name__, logging.ERROR, log.DefaultLoggingHandler())


def get_args():
    """Get the args parsed from the command line and does arg handling stuff"""

    arg = argparse.ArgumentParser(description="Generate and switch lockscreen images")

    arg.add_argument(
        "-v", "--version", action="store_true", help="Display current version"
    )
    arg.add_argument("-g", "--generate", action="store_true", help="Generate themes")
    arg.add_argument(
        "-i", "--input", metavar='"path/to/dir"', help="The input file or directory"
    )
    arg.add_argument("-r", "--radius", metavar="radius", help="Radius for the blur")
    arg.add_argument("--verbose", action="store_true", help="Verbose logging")
    arg.add_argument(
        "-b",
        "--brightness",
        metavar="brightness",
        help="The brightness of the image (darker < 1.0 < lighter)",
    )
    arg.add_argument(
        "--override", action="store_true", help="Override exisiting lockscreen file"
    )
    arg.add_argument(
        "--clear", action="store_true", help="Clear all data relating to JYOU"
    )
    arg.add_argument("--progress", action="store_true", help="Display progress")

    return arg


def parse_args(parser):
    """
    Parses the arguments onto different actions

    Arguments:
            parser (idk) -- the argument parser
    """

    args = parser.parse_args()

    if len(sys.argv) <= 1:
        parser.print_help()
        sys.exit(1)

    if args.version:
        print(f"JYOU {__version__} by {__author__} <{__email__}>")
        sys.exit(0)

    verbose_logging = bool(args.verbose)
    log_level = get_log_level(verbose_logging)
    logger.setLevel(log_level)

    if args.input:
        blur_strength = config_handler.compare_flag_with_config(
            args.radius, config_handler.parse_config()["blur"]
        )
        brightness = config_handler.compare_flag_with_config(
            args.brightness, config_handler.parse_config()["brightness"]
        )
        progress = config_handler.compare_flag_with_config(
            args.progress, config_handler.parse_config()["progress"]
        )
        output_path = config_handler.parse_config()["out_directory"]

        generator = LockscreenGenerator(
            args.input,
            progress_bar=progress,
            verbose_logging=verbose_logging,
            override=args.override,
            blur_strength=blur_strength,
            brightness=brightness,
            output_path=output_path,
        )

        if args.generate:
            generator.generate()
        else:
            generator.update()

    elif args.clear:
        clear = input(
            "Are you sure you want to remove the cache relating to JYOU? [y/N] "
        ).lower()
        if clear == "y":
            shutil.rmtree(DATA_PATH)
            logger.info("Cleared JYOU cache folders")
        else:
            logger.warning("Canceled clearing cache folders...")


def main():
    # Create required directories
    os.makedirs(DATA_PATH, exist_ok=True)
    os.makedirs(CONFIG_PATH, exist_ok=True)

    parser = get_args()
    parse_args(parser)


def get_log_level(verbose_logging):
    if verbose_logging:
        return logging.INFO

    return logging.WARNING


if __name__ == "__main__":
    main()



"""The main generation file containing tools to generate the needed images"""
import os
import subprocess
import re
import random
import sys
import logging
from typing import List, Tuple

import tqdm
from PIL import Image, ImageFilter, ImageEnhance

from .settings import DATA_PATH, DEBUG_MODE
from . import utils, log

logger = log.setup_logger(
    __name__ + "default", logging.WARN, log.DefaultLoggingHandler()
)
tqdm_logger = log.setup_logger(
    __name__ + ".tqdm", logging.WARN, log.TqdmLoggingHandler()
)


# pylint: disable=too-many-instance-attributes
class LockscreenGenerator:
    """Main class for anything to do with lockscreen"""

    def __init__(self, image_path, **kwargs):
        """
        The initialisation method

        Arguments:
            image_path (str): location to the image ('/home/bob/pic.png')
        """
        self.progress_bar = kwargs.get("progress_bar", False)
        self.verbose_logging = kwargs.get("verbose_logging", False)
        self.override = kwargs.get("override", False)
        self.blur_strength = kwargs.get("blur_strength", 0)
        self.brightness = kwargs.get("brightness", 1)
        self.out_dir = kwargs.get("output_path", DATA_PATH)

        self.resolutions = get_resolution_image()
        self.screen_md5 = utils.md5(
            ",".join(str(i) for j in self.resolutions for i in j)
        )[:20]

        if DEBUG_MODE:
            logger.setLevel(logging.DEBUG)
            tqdm_logger.setLevel(logging.DEBUG)
        elif self.verbose_logging:
            logger.setLevel(logging.INFO)
            tqdm_logger.setLevel(logging.INFO)

        os.makedirs(self.out_dir, exist_ok=True)

        self.image_paths = get_image_path_list(image_path)
        if not self.image_paths:
            logger.critical("File does not exist!")
            sys.exit(1)

    def generate(self):
        """Generate the lockscreen image"""
        # Apply this function to every image in the passed list
        non_generated_images = []
        lockscreen_dir = os.path.join(self.out_dir, "lockscreen")
        os.makedirs(lockscreen_dir, exist_ok=True)

        for image_path in self.image_paths:
            out_path = get_out_path_from_md5(
                image_path, self.screen_md5, lockscreen_dir
            )

            if not os.path.isfile(out_path) or self.override:
                non_generated_images.append(
                    {"image_path": image_path, "out_path": out_path}
                )

        if len(non_generated_images) > 0:
            for i in tqdm.tqdm(
                range(len(non_generated_images)),
                bar_format=log.BAR_FORMAT,
                disable=not self.progress_bar,
            ):
                # Save the image
                out_path = non_generated_images[i]["out_path"]
                lockscreen_image = generate_lockscreen_image(
                    non_generated_images[i]["image_path"],
                    self.resolutions,
                    self.blur_strength,
                    self.brightness,
                )
                lockscreen_image.save(out_path)
        else:
            logger.info("No lockscreens to generate.")

    def update(self):
        """Update the wallpaper based on the parsed image in the parent class"""
        lockscreen_dir = os.path.join(self.out_dir, "lockscreen")
        image_path = get_random_image_path(self.image_paths)
        self.image_paths = [image_path]

        image_out_path = get_out_path_from_md5(
            image_path, self.screen_md5, lockscreen_dir
        )
        # Copy the image if it exists
        if os.path.isfile(image_out_path):
            symlink_path = os.path.join(self.out_dir, "current_lockscreen.png")
            symlink_image(image_out_path, symlink_path)

            # Run postscripts
            utils.run_hooks()
        else:
            self.generate()
            self.update()


def get_resolution_image() -> List[Tuple[int]]:
    """Gets the screen resolution"""
    display_re = r"([0-9]+)x([0-9]+)\+([0-9]+)\+([0-9]+)"  # Regex to find the monitor resolutions

    command_output = subprocess.check_output(["xrandr"])
    resolution_list = [
        [int(j) for j in i] for i in re.findall(display_re, str(command_output))
    ]

    return [tuple(i) for i in resolution_list]


def get_out_path_from_md5(image_path: str, screen_md5: str, out_directory: str) -> str:
    """
    Gets the out path path from the image and screen md5

    Arguments:
        image_path (str):       the path to the image
        screen_md5 (str):       the md5 of the screen
        out_directory (str):    the directory to append to the start of the path

    Returns:
        (str): the generated path
    """
    image_md5 = utils.md5_file(image_path)[:20]
    return os.path.join(out_directory, f"{image_md5}_{screen_md5}.png")


def generate_lockscreen_image(
    image_path: str, resolutions: List[Tuple[int]], blur: int, brightness: float
) -> Image:
    """
    Generates the image for the lockscreen

    Arguments:
        image_path (str): the path to the image
        resolutions (List[tuple]): the resolutions of the screens to generate for
        blur (int): the strength of the blur to be applied
        brightness (float): how bright the image should be

    Returns:
        (PIL.Image): the raw generated image
    """
    screens = []

    image = Image.open(image_path).convert("RGB")
    output_image_width, output_image_height = get_accumulative_dimensions(resolutions)

    # Repeat for every screen the user has
    for resolution in resolutions:
        dimensions = get_resolution_dimensions(resolution)
        resolution_image = crop_image_to_dimensions(image, dimensions)
        if blur:
            resolution_image = blur_image(resolution_image, blur)
        screens.append([resolution_image, get_resolution_offset(resolution)])

    output_image = Image.new(
        "RGB", (output_image_width, output_image_height), (0, 0, 0)
    )

    # Add the images in their locations onto the new image
    for screen in screens:
        output_image.paste(*screen)

    if brightness:
        enhancer = ImageEnhance.Brightness(output_image)
        output_image = enhancer.enhance(brightness)

    return output_image


def get_resolution_dimensions(resolution: Tuple[int]) -> Tuple[int]:
    """
    Gets the width and height from the specified resolution in the form of
    [width, height, offset_x, offset_y]

    Arguments:
        resolution (Tuple[int]): the resolution

    Returns:
        (Tuple[int]): the width and height
    """
    width, height, _, _ = map(int, resolution)
    return (width, height)


def get_resolution_offset(resolution: Tuple[int]) -> Tuple[int]:
    """
    Gets the x offset and y offset from the specified resolution in the form of
    [width, height, offset_x, offset_y]

    Arguments:
        resolution (Tuple[int]): the resolution

    Returns:
        (Tuple[int]): the x offset and the y offset
    """
    _, _, screen_x, screen_y = map(int, resolution)
    return (screen_x, screen_y)


def get_accumulative_dimensions(resolutions: List[Tuple[int]]) -> Tuple[int]:
    """
    Gets the accumlative dimensions of the given resolutions

    Arguments:
        resolutions (List[Tuple[int]]): the list of resolutions

    Returns:
        (Tuple[int]): the width and height of the accumlative dimensions
    """
    output_width, output_height = (0, 0)
    for resolution in resolutions:
        resolution_width, resolution_height = get_resolution_dimensions(resolution)
        resolution_x, resolution_y = get_resolution_offset(resolution)

        if output_width < resolution_width + resolution_x:
            output_width = resolution_width + resolution_x

        if output_height < resolution_height + resolution_y:
            output_height = resolution_height + resolution_y

    return (output_width, output_height)


def crop_image_to_dimensions(image: Image, dimensions: Tuple[int]) -> Image:
    """
    Crops the image to the specified dimensions

    Arguments:
        image (PIL.Image): the image to be cropped
        dimensions (Tuple[int]): the width and height to be cropped to

    Returns:
        (PIL.Image): the cropped image
    """
    image_width, image_height = image.size
    width, height = dimensions

    ratio = min(image_width / width, image_height / height)
    ratio_dimensions = (int(image_width / ratio), int(image_height / ratio))
    resized_image = image.resize(ratio_dimensions, Image.LANCZOS)

    crop_box = (
        (ratio_dimensions[0] - width) / 2,
        (ratio_dimensions[1] - height) / 2,
        (ratio_dimensions[0] + width) / 2,
        (ratio_dimensions[1] + height) / 2,
    )

    cropped_image = resized_image.crop(crop_box)

    return cropped_image


def blur_image(image: Image, blur: int) -> Image:
    """
    Blur the image by the strength given

    Arguments:
        image (PIL.Image): the image to be blurred
        blur (int): the strength to blur the image

    Returns:
        (PIL.Image): the blurred image
    """
    if int(blur) != 0:
        image = image.filter(ImageFilter.GaussianBlur(int(blur)))

    return image


def get_image_path_list(image_directory: str) -> List[str]:
    """
    Gets the absolute image paths within a directory

    Arguments:
        image_directory (str): the path to the image or the directory

    Returns:
        (List[str]): the list of absolute paths to images
    """
    if os.path.isfile(image_directory):
        return [utils.get_absolute_image_path(image_directory)]

    return [
        utils.get_absolute_image_path(os.path.join(image_directory, img))
        for img in utils.get_directory_image_paths(image_directory)
    ]


def get_random_image_path(image_paths: List[str]) -> str:
    """Gets a random path to a image from a given list

    Arguments:
        image_paths (List[str]): a list containing paths to images

    Returns:
        (str): a randomly picked image path
    """
    random.shuffle(image_paths)
    return image_paths[0]


def symlink_image(image_path: str, symlink_path: str):
    """
    Symlink an image to a given path

    Arguments:
        image_path (str):   the path to the original image
        symlink_path(str):  the path to symlink to
    """
    if os.path.isfile(symlink_path):
        os.remove(symlink_path)

    os.symlink(image_path, symlink_path)



from setuptools import setup
from jyou.__init__ import __author__, __email__, __version__

LONG_DESC = open("README.md").read()

setup(
    name="JYOU",
    version=__version__,
    description="Simple lockscreen manager for tiling window managers. Generates a lockscreen for multiscreened i3locks",
    long_description_content_type="text/markdown",
    long_description=LONG_DESC,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: X11 Applications",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.10",
    ],
    url="http://github.com/slapelachie/jyou",
    author=__author__,
    author_email=__email__,
    license="GPLv3",
    packages=["jyou"],
    entry_points={"console_scripts": ["jyou=jyou.__main__:main"]},
    install_requires=[
        "Pillow",
        "tqdm",
    ],
    zip_safe=False,
)



"""An assortment of utilities to aid this project"""
import hashlib
import os
import re
import subprocess

from typing import List

from .settings import CONFIG_PATH


def md5(string: str) -> str:
    """
    Generates a md5 hash based on the parsed string

    Arguments:
        string (str): a string to be encoded

    Returns:
        (str): md5 encoding of the given string
    """
    hash_md5 = hashlib.md5(str(string).encode())
    return hash_md5.hexdigest()


def md5_file(file_path: str) -> str:
    """
    Generates a md5 hash based on the file parsed

    Arguments:
        file_path (str): location of the file ('/home/bob/pic.png')

    Returns:
        (str): md5 encoding of the given file
    """
    hash_md5 = hashlib.md5()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def get_absolute_image_path(image_path: str) -> str:
    """
    Get the absolute path of a parsed file (image)

    Arguments:
        image_path (str): location of the file

    Returns:
        (str): the absolute path of the parsed file
    """
    if os.path.isfile(image_path):
        return os.path.abspath(image_path)

    return None


def get_directory_image_paths(image_directory: str) -> List[str]:
    """
    Get a list of all image paths in a directory

    Arguments:
        img_dir (str): the directory where the images are stored

    Returns:
        (List[str]): list of all file paths in a directory
    """
    file_types = ("png", "jpg", "jpeg")
    return [
        img.name
        for img in os.scandir(image_directory)
        if img.name.lower().endswith(file_types)
    ]


def run_hooks():
    """Run all scripts within the hooks folder"""
    hooks_dir = os.path.join(CONFIG_PATH, "hooks")
    os.makedirs(hooks_dir, exist_ok=True)

    scripts = [
        f
        for f in os.listdir(hooks_dir)
        if re.match(r"^([0-9]{2}-\w+)", f)
        and os.access(os.path.join(hooks_dir, f), os.X_OK)
    ]
    scripts.sort()

    for script in scripts:
        script = os.path.join(hooks_dir, script)

        with open(os.devnull, "w", encoding="UTF-8") as devnull:
            subprocess.run(script, stdout=devnull, stderr=subprocess.STDOUT, check=True)
